#pragma once
#include <catch.hpp>
#include <panda/protocol/http.h>

using namespace panda;
using namespace panda::protocol::http;
using Catch::Matchers::StartsWith;

using panda::date::Date;
using Method = Request::Method;
